
module Test.UtilSpec where

import Data.Map.Strict qualified as Map
import Data.Map.Strict.Patch.Internal
import Data.Monoid
import Data.Patch.Class
import Test.Hspec
import Test.QuickCheck
import Test.Util


prop_creates_applicable_patches :: Doc -> Property
prop_creates_applicable_patches d =
  forAll (patchesFrom d) $ \p -> isV $ applicable p d

prop_creates_patches_that_are_right_composable_with_mempty :: Doc -> Property
prop_creates_patches_that_are_right_composable_with_mempty d =
  forAll (patchesFrom d) $ \p -> isV $ composable mempty p

prop_creates_patches_that_are_left_composable_with_mempty :: Doc -> Property
prop_creates_patches_that_are_left_composable_with_mempty d =
  forAll (patchesFrom d) $ \p -> isV $ composable p mempty

prop_creates_patches_that_are_applicable_in_sequence :: Doc -> Property
prop_creates_patches_that_are_applicable_in_sequence d =
  forAll (historyFrom d 2) $
    \[p, q] -> isV $ applicable p d <> applicable q (act p d)

prop_creates_patches_that_are_applicable_in_merged_form :: Doc -> Property
prop_creates_patches_that_are_applicable_in_merged_form d =
  forAll (historyFrom d 2) $
    \[p, q] -> isV $ applicable (p <> q) d

prop_creates_patches_that_are_composable :: Doc -> Property
prop_creates_patches_that_are_composable d =
  forAll (historyFrom d 2) $
    \[p, q] -> isV $ composable p q


spec_history_from :: Spec
spec_history_from = describe "historyFrom" $ do
  it "creates patches that are applicable in merged form" $ do
    property $ \d ->
      forAll (historyFrom d 2) $ \[p, q] ->
        isV $ applicable (p <> q) d

  it "creates patches that are composable" $ do
    property $ \d ->
      forAll (historyFrom d 2) $ \[p, q] -> isV $ composable p q

   -- it "check simple act" $ do
    --   let d = fromList [ (3, 5) ]
    --   let p = PatchMap $ Map.fromList [ (3, Replace { _old = 5, _new = Just 3 })]

    --   (act p d) `shouldBe` (Map.fromList [ (3, 3 )])


--     it "check problematic patch" $ do
--       let d = fromList [ (3, 5), (-2, -2) ]
--       let d = [ PatchMap (fromList [(3,Replace {_old = Nothing, _new = Just 5})])
--               , PatchMap (fromList [(-2,Replace {_old = Nothing, _new = Just (-2)}),(3,Replace {_old = Just
-- 5, _new = Just 1})])]
--       let [p, q] = historyFrom d 2

--       isV $ applicable p d <> applicable q (act p d)

